@@ -12,7 +12,6 @@ import click
 import pandas as pd
 import numpy as np
 import hail as hl
-from bokeh.io.export import get_screenshot_as_png
 from bokeh.resources import CDN
 from bokeh.embed import file_html
 from bokeh.transform import factor_cmap, factor_mark
@@ -20,7 +19,7 @@ from bokeh.plotting import ColumnDataSource, figure
 from bokeh.palettes import turbo  # pylint: disable=no-name-in-module
 from bokeh.models import CategoricalColorMapper, HoverTool
 
-from joint_calling import utils
+from joint_calling import utils, resources
 from joint_calling import _version
 
 
@@ -61,6 +60,12 @@ logger.setLevel(logging.INFO)
     required=True,
     callback=utils.get_validation_callback(ext='ht', must_exist=True),
 )
+@click.option(
+    '--meta-tsv',
+    'meta_tsv_path',
+    required=True,
+    callback=utils.get_validation_callback(ext='tsv', must_exist=True),
+)
 @click.option('--out-path-pattern', 'out_path_pattern')
 @click.option(
     '--hail-billing',
@@ -74,10 +79,13 @@ def main(  # pylint: disable=too-many-arguments,too-many-locals,missing-function
     loadings_ht_path: str,
     provided_pop_ht_path: str,
     inferred_pop_ht_path: str,
+    meta_tsv_path: str,
     out_path_pattern: Optional[str],
     hail_billing: str,  # pylint: disable=unused-argument
 ):
-    utils.init_hail(__file__)
+    local_tmp_dir = utils.init_hail(__file__)
+
+    meta_ht = utils.parse_input_metadata(meta_tsv_path, local_tmp_dir)
 
     produce_plots(
         eigenvalues_path=eigenvalues_path,
@@ -85,62 +93,98 @@ def main(  # pylint: disable=too-many-arguments,too-many-locals,missing-function
         loadings_ht_path=loadings_ht_path,
         provided_pop_ht_path=provided_pop_ht_path,
         inferred_pop_ht_path=inferred_pop_ht_path,
+        meta_ht=meta_ht,
         out_path_pattern=out_path_pattern,
     )
 
 
+def key_by_external_id(ht, sample_map_ht):
+    """
+    Assuming ht.s is a CPG id, replaces it with external ID, using map in sample_map_ht
+    """
+    ht = ht.annotate(internal_id=ht.s).key_by('internal_id')
+    ht = (
+        ht.annotate(
+            s=hl.if_else(
+                hl.is_defined(sample_map_ht[ht.internal_id]),
+                sample_map_ht[ht.internal_id].external_id,
+                ht.internal_id,
+            )
+        )
+        .key_by('s')
+        .drop('internal_id')
+    )
+    return ht
+
+
 def produce_plots(
     eigenvalues_path: str,
     scores_ht_path: str,
     loadings_ht_path: str,
     provided_pop_ht_path: str,
     inferred_pop_ht_path: str,
+    meta_ht: hl.Table,
     out_path_pattern: Optional[str] = None,
     number_of_pcs: Optional[int] = None,
 ):
     """
-    Generate plots in png and html formats, write for each PC (of n_pcs) and
+    Generate plots in HTML format, write for each PC (of n_pcs) and
     scope ("study", "continental_pop", "subpop", plus for loadings) into
     file paths defined by `out_path_pattern`.
     """
-    scores = hl.read_table(scores_ht_path)
-    sample_names = scores.s.collect()
+    scores_ht = hl.read_table(scores_ht_path)
     provided_pop_ht = hl.read_table(provided_pop_ht_path)
     inferred_pop_ht = hl.read_table(inferred_pop_ht_path)
-    scores = scores.annotate(
+
+    sample_map_ht = meta_ht.select('external_id')
+    scores_ht = key_by_external_id(scores_ht, sample_map_ht).cache()
+    provided_pop_ht = key_by_external_id(provided_pop_ht, sample_map_ht).cache()
+    inferred_pop_ht = key_by_external_id(inferred_pop_ht, sample_map_ht).cache()
+
+    scores_ht = scores_ht.annotate(
         continental_pop=hl.case()
         .when(
-            provided_pop_ht[scores.s].continental_pop != '',
-            provided_pop_ht[scores.s].continental_pop,
+            provided_pop_ht[scores_ht.s].continental_pop != '',
+            provided_pop_ht[scores_ht.s].project
+            + ' ['
+            + provided_pop_ht[scores_ht.s].continental_pop
+            + ']',
         )
         .default(
-            inferred_pop_ht[scores.s].pop
-            + ' ['
-            + provided_pop_ht[scores.s].project
-            + ', inferred]'
+            provided_pop_ht[scores_ht.s].project
+            + ' [inferred '
+            + inferred_pop_ht[scores_ht.s].pop
+            + ']'
         ),
         subpop=hl.case()
-        .when(provided_pop_ht[scores.s].subpop != '', provided_pop_ht[scores.s].subpop)
-        .default(
-            inferred_pop_ht[scores.s].pop
+        .when(
+            provided_pop_ht[scores_ht.s].subpop != '',
+            provided_pop_ht[scores_ht.s].project
             + ' ['
-            + provided_pop_ht[scores.s].project
-            + ', inferred]'
+            + provided_pop_ht[scores_ht.s].subpop
+            + ']',
+        )
+        .default(
+            provided_pop_ht[scores_ht.s].project
+            + ' [inferred '
+            + inferred_pop_ht[scores_ht.s].pop
+            + ']'
         ),
-        study=provided_pop_ht[scores.s].project,
-    )
+        study=provided_pop_ht[scores_ht.s].project,
+    ).cache()
 
-    eigenvalues = hl.import_table(
+    eigenvalues_ht = hl.import_table(
         eigenvalues_path, no_header=True, types={'f0': hl.tfloat}
     ).f0.collect()
-    eigenvalues = pd.to_numeric(eigenvalues)
-    variance = np.divide(eigenvalues[1:], float(eigenvalues.sum())) * 100
+    eigenvalues_ht = pd.to_numeric(eigenvalues_ht)
+    variance = np.divide(eigenvalues_ht[1:], float(eigenvalues_ht.sum())) * 100
     variance = variance.round(2)
-    number_of_pcs = number_of_pcs or len(eigenvalues) - 1
+    number_of_pcs = number_of_pcs or len(eigenvalues_ht) - 1
 
     plots = []
 
-    studies = scores.study.collect()
+    sample_names = scores_ht.s.collect()
+    studies = scores_ht.study.collect()
     unique_studies = remove_duplicates(studies)
     bg_studies = [s for s in unique_studies if s == 'gnomad']
     fg_studies = [s for s in unique_studies if s != 'gnomad']
@@ -149,9 +193,9 @@ def produce_plots(
         _plot_study(
             number_of_pcs,
             variance,
-            scores,
+            scores_ht,
+            studies,
             sample_names,
-            unique_studies,
             bg_studies,
             fg_studies,
             out_path_pattern=out_path_pattern,
@@ -162,7 +206,8 @@ def produce_plots(
         _plot_continental_pop(
             number_of_pcs,
             variance,
-            scores,
+            scores_ht,
+            studies,
             sample_names,
             bg_studies,
             fg_studies,
@@ -174,7 +219,8 @@ def produce_plots(
         _plot_subcontinental_pop(
             number_of_pcs,
             variance,
-            scores,
+            scores_ht,
+            studies,
             sample_names,
             bg_studies,
             fg_studies,
@@ -202,7 +248,6 @@ def _plot_study(
     out_path_pattern=None,
 ):
     tooltips = [('labels', '@label'), ('samples', '@samples')]
-    # plot by study
     cntr: Counter = Counter(studies)
     labels = [f'{x} ({cntr[x]})' for x in studies]
     unique_labels = list(Counter(labels).keys())
@@ -216,6 +261,7 @@ def _plot_study(
             x_axis_label=f'PC{pc1 + 1} ({variance[pc1]})%)',
             y_axis_label=f'PC{pc2 + 1} ({variance[pc2]}%)',
             tooltips=tooltips,
+            width=1000,
         )
         source = ColumnDataSource(
             dict(
@@ -223,6 +269,7 @@ def _plot_study(
                 y=scores.scores[pc2].collect(),
                 label=labels,
                 samples=sample_names,
+                study=studies,
             )
         )
         plot.scatter(
@@ -242,9 +289,6 @@ def _plot_study(
         plot.add_layout(plot.legend[0], 'left')
         plots.append(plot)
         if out_path_pattern:
-            plot_filename = out_path_pattern.format(scope='study', pci=pc2, ext='png')
-            with hl.hadoop_open(plot_filename, 'wb') as f:
-                get_screenshot_as_png(plot).save(f, format='PNG')
             html = file_html(plot, CDN, 'my plot')
             plot_filename_html = out_path_pattern.format(
                 scope='study', pci=pc2, ext='html'
@@ -258,6 +302,7 @@ def _plot_continental_pop(
     number_of_pcs,
     variance,
     scores,
+    studies,
     sample_names,
     bg_studies,
     fg_studies,
@@ -278,6 +323,7 @@ def _plot_continental_pop(
             x_axis_label=f'PC{pc1 + 1} ({variance[pc1]})%)',
             y_axis_label=f'PC{pc2 + 1} ({variance[pc2]}%)',
             tooltips=tooltips,
+            width=1000,
         )
         source = ColumnDataSource(
             dict(
@@ -285,6 +331,7 @@ def _plot_continental_pop(
                 y=scores.scores[pc2].collect(),
                 label=labels,
                 samples=sample_names,
+                study=studies,
             )
         )
         plot.scatter(
@@ -297,18 +344,13 @@ def _plot_continental_pop(
                 bg_studies + fg_studies,
             ),
             source=source,
-            size=2,
+            size=4,
             color=factor_cmap('label', turbo(len(unique_labels)), unique_labels),
             legend_group='label',
         )
         plot.add_layout(plot.legend[0], 'left')
         plots.append(plot)
         if out_path_pattern:
-            plot_filename = out_path_pattern.format(
-                scope='continental_pop', pci=pc2, ext='png'
-            )
-            with hl.hadoop_open(plot_filename, 'wb') as f:
-                get_screenshot_as_png(plot).save(f, format='PNG')
             html = file_html(plot, CDN, 'my plot')
             plot_filename_html = out_path_pattern.format(
                 scope='continental_pop', pci=pc2, ext='html'
@@ -322,6 +364,7 @@ def _plot_subcontinental_pop(
     number_of_pcs,
     variance,
     scores,
+    studies,
     sample_names,
     bg_studies,
     fg_studies,
@@ -342,6 +385,7 @@ def _plot_subcontinental_pop(
             x_axis_label=f'PC{pc1 + 1} ({variance[pc1]})%)',
             y_axis_label=f'PC{pc2 + 1} ({variance[pc2]}%)',
             tooltips=tooltips,
+            width=1000,
         )
         source = ColumnDataSource(
             dict(
@@ -349,6 +393,7 @@ def _plot_subcontinental_pop(
                 y=scores.scores[pc2].collect(),
                 label=labels,
                 samples=sample_names,
+                study=studies,
             )
         )
         plot.scatter(
@@ -361,16 +406,13 @@ def _plot_subcontinental_pop(
                 bg_studies + fg_studies,
             ),
             source=source,
-            size=2,
+            size=4,
             color=factor_cmap('label', turbo(len(unique_labels)), unique_labels),
             legend_group='label',
         )
         plot.add_layout(plot.legend[0], 'left')
         plots.append(plot)
         if out_path_pattern:
-            plot_filename = out_path_pattern.format(scope='subpop', pci=pc2, ext='png')
-            with hl.hadoop_open(plot_filename, 'wb') as f:
-                get_screenshot_as_png(plot).save(f, format='PNG')
             html = file_html(plot, CDN, 'my plot')
             plot_filename_html = out_path_pattern.format(
                 scope='subpop', pci=pc2, ext='html'
@@ -383,19 +425,24 @@ def _plot_subcontinental_pop(
 def _plot_loadings(number_of_pcs, loadings_ht_path, out_path_pattern=None):
     loadings_ht = hl.read_table(loadings_ht_path)
     plots = []
+    gtf_ht = hl.experimental.import_gtf(
+        resources.GENCODE_GTF,
+        reference_genome='GRCh38',
+        skip_invalid_contigs=True,
+        min_partitions=12,
+        force_bgz=True,
+    )
     for i in range(number_of_pcs - 1):
         pc = i + 1
         plot = manhattan_loadings(
-            pvals=hl.abs(loadings_ht.loadings[i]),
-            locus=loadings_ht.locus,
+            iteration=i,
+            gtf=gtf_ht,
+            loadings=loadings_ht,
             title='Loadings of PC ' + str(pc),
             collect_all=True,
         )
         plots.append(plot)
         if out_path_pattern:
-            plot_filename = out_path_pattern.format(scope='loadings', pci=pc, ext='png')
-            with hl.hadoop_open(plot_filename, 'wb') as f:
-                get_screenshot_as_png(plot).save(f, format='PNG')
             html = file_html(plot, CDN, 'my plot')
             plot_filename_html = out_path_pattern.format(
                 scope='loadings', pci=pc, ext='html'
@@ -406,8 +453,9 @@ def _plot_loadings(number_of_pcs, loadings_ht_path, out_path_pattern=None):
 
 
 def manhattan_loadings(
-    pvals,
-    locus=None,
+    iteration,
+    gtf,
+    loadings,
     title=None,
     size=4,
     hover_fields=None,
@@ -427,15 +475,17 @@ def manhattan_loadings(
         '#bcbd22',
         '#17becf',
     ]
-    if locus is None:
-        locus = pvals._indices.source.locus  # pylint: disable=protected-access
 
-    ref = locus.dtype.reference_genome
+    # add gene names, p-values, and locus info
+    loadings = loadings.annotate(gene_names=gtf[loadings.locus].gene_name)
+    pvals = hl.abs(loadings.loadings[iteration])
+    locus = loadings.locus
 
     if hover_fields is None:
         hover_fields = {}
 
     hover_fields['locus'] = hl.str(locus)
+    hover_fields['gene'] = hl.str(loadings.gene_names)
 
     source_pd = (
         hl.plot.plots._collect_scatter_plot_data(  # pylint: disable=protected-access
@@ -449,6 +499,7 @@ def manhattan_loadings(
     source_pd['_contig'] = [locus.split(':')[0] for locus in source_pd['locus']]
 
     observed_contigs = set(source_pd['_contig'])
+    ref = locus.dtype.reference_genome
     observed_contigs = [
         contig for contig in ref.contigs.copy() if contig in observed_contigs
     ]
